"""
Holiday Features Module

This module provides functions to add holiday-related features to pandas DataFrames.
"""

import pandas as pd
from typing import List, Optional, Union
import ast
from pathlib import Path

# Constants
VALID_COUNTRIES = ['ES', 'IT']
SPAIN_COUNTIES = [
    'ES-AN', 'ES-AR', 'ES-AS', 'ES-CB', 'ES-CE', 'ES-CL', 'ES-CM', 'ES-CN',
    'ES-CT', 'ES-EX', 'ES-GA', 'ES-IB', 'ES-MC', 'ES-MD', 'ES-ML', 'ES-NC',
    'ES-PV', 'ES-RI', 'ES-VC'
]

def _load_holiday_data(country_code: str) -> pd.DataFrame:
    """
    Load holiday data for a specific country.
    
    Parameters
    ----------
    country_code : str
        The country code ('ES' for Spain, 'IT' for Italy)
    
    Returns
    -------
    pd.DataFrame
        DataFrame containing holiday information
    
    Raises
    ------
    ValueError
        If country_code is not supported
    FileNotFoundError
        If holiday data file is not found
    """
    if country_code not in VALID_COUNTRIES:
        raise ValueError(f"Country code {country_code} not supported. Valid codes: {VALID_COUNTRIES}")
    
    # Get the directory where this script is located
    current_dir = Path(__file__).parent.parent
    
    if country_code == 'ES':
        file_path = current_dir / 'db' / 'spain_holidays.csv'
        holidays_df = pd.read_csv(file_path)
        # Convert string representation of list to actual list
        holidays_df['counties'] = holidays_df['counties'].apply(
            lambda x: ast.literal_eval(x) if pd.notna(x) and x != '' else []
        )
        return holidays_df
    else:  # IT
        # TODO: Implement Italy holidays when available
        return pd.DataFrame(columns=['date', 'global', 'counties'])

def add_holiday_features(
    df: pd.DataFrame,
    date_column: str = 'date',
    country_column: str = 'country',
    county_column: Optional[str] = 'county',
    county_threshold: int = 3
) -> pd.DataFrame:
    """
    Add holiday-related features to a pandas DataFrame based on date, country, and county information.
    
    Parameters
    ----------
    df : pd.DataFrame
        Input DataFrame containing date and country information
    date_column : str, default='date'
        Name of the column containing datetime values
    country_column : str, default='country'
        Name of the column containing country codes
    county_column : str, optional, default='county'
        Name of the column containing county codes
    county_threshold : int, default=3
        Number of counties threshold for many_counties_holiday flag
    
    Returns
    -------
    pd.DataFrame
        DataFrame with added holiday features:
        - is_public_holiday: True if the date is a global holiday
        - is_local_holiday: True if the date is a holiday for the specific county
        - many_counties_holiday: True if more than county_threshold counties celebrate that holiday
    
    Raises
    ------
    ValueError
        If required columns are missing or contain invalid values
    """
    # Input validation
    if date_column not in df.columns:
        raise ValueError(f"Date column '{date_column}' not found in DataFrame")
    if country_column not in df.columns:
        raise ValueError(f"Country column '{country_column}' not found in DataFrame")
    if county_threshold < 1:
        raise ValueError("county_threshold must be a positive integer")
    
    # Make a copy to avoid modifying the input DataFrame
    result_df = df.copy()
    
    # Ensure date column is datetime
    result_df[date_column] = pd.to_datetime(result_df[date_column])
    
    # Validate country codes
    invalid_countries = set(result_df[country_column].unique()) - set(VALID_COUNTRIES)
    if invalid_countries:
        raise ValueError(f"Invalid country codes found: {invalid_countries}. Valid codes: {VALID_COUNTRIES}")
    
    # Initialize holiday columns
    result_df['is_public_holiday'] = False
    result_df['is_local_holiday'] = False
    result_df['many_counties_holiday'] = False
    
    # Process each country separately
    for country in VALID_COUNTRIES:
        country_mask = result_df[country_column] == country
        if not country_mask.any():
            continue
            
        holidays_df = _load_holiday_data(country)
        
        # Convert holiday dates to datetime for comparison
        holidays_df['date'] = pd.to_datetime(holidays_df['date'])
        
        # Process each date in the holidays DataFrame
        for _, holiday in holidays_df.iterrows():
            date_mask = result_df[date_column].dt.date == holiday['date'].date()
            combined_mask = country_mask & date_mask
            
            if not combined_mask.any():
                continue
            
            # Set public holiday flag
            if holiday.get('global', False):
                result_df.loc[combined_mask, 'is_public_holiday'] = True
            
            # Set local holiday flag if county matches
            counties = holiday.get('counties', [])
            if counties and county_column and county_column in result_df.columns:
                local_mask = result_df[county_column].isin(counties)
                result_df.loc[combined_mask & local_mask, 'is_local_holiday'] = True
            
            # Set many counties flag based on threshold
            if len(counties) > county_threshold:
                result_df.loc[combined_mask, 'many_counties_holiday'] = True
    
    return result_df
